#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging
from argparse import Namespace
from pathlib import Path
from typing import Optional

import click
import yaml

logger = logging.getLogger(__name__)


class Configuration(Namespace):
    def __init__(self):
        super(Configuration, self).__init__()
        self.opts: Optional[str] = None

        self.outdir: str = None
        self.link: str = None

        self.verbose: int = 0
        self.unclean: bool = False
        self.move_media: bool = False
        self.move_html: bool = False

    def process_click_arguments(self, context: click.Context):
        if context.params.get("opts") is not None:
            self.parse_yaml_options(context.params["opts"])
        for arg_key in context.params.keys():
            if not hasattr(self, arg_key):
                logger.warning(f"Ignoring an unknown CLI argument: {arg_key}")
                continue
            val = context.params[arg_key]
            if val is None or val == ():
                # don't overwrite with an empty value
                continue
            setattr(self, arg_key, val)

    def parse_yaml_options(self, file_path: str):
        yaml_file_loc = Path(file_path)
        if not yaml_file_loc.exists():
            logger.error(f"No YAML file found at {yaml_file_loc}")
            return
        with yaml_file_loc.open() as file:
            try:
                opts = yaml.safe_load(file)
            except yaml.YAMLError as e:
                logger.error(f"Could not parse YAML options file: {e}")
                return
        for arg_key, val in opts.items():
            if not hasattr(self, arg_key):
                logger.warning(f"Ignoring an unknown YAML argument: {arg_key}")
                continue
            setattr(self, arg_key, val)



import logging

import requests

logger = logging.getLogger(__name__)


# Get subreddit name from post
def get_sub_from_post(post):
    if post.get("subreddit") is None:
        link = post["permalink"]
        post["subreddit"] = link.split("/")[2]
    return post


# Recover deleted posts via pushshift
# Request a specific post to be recovered
def recover_deleted_post(post):
    if post["selftext"] == "[deleted]":
        try:
            response = requests.get(
                "https://api.pushshift.io/reddit/submission/search?ids={id}".format(
                    id=post["id"]
                )
            )
            data = response.json()["data"]
            logger.debug(data)
            logger.debug(len(data))
            if len(data) == 1:
                recovered_post = data[0]
                post["selftext"] = recovered_post["selftext"]
                post["author"] = recovered_post["author"]
                post["url"] = recovered_post["url"]
                post["recovered"] = True
                logging.info(f"Recovered {post.get('id', '')} from pushshift")
        except Exception as e:
            logging.error(e)
        return post
    else:
        return post


# Request a specific comment to be recovered
def recover_deleted_comment(comment):
    try:
        response = requests.get(
            "https://api.pushshift.io/reddit/comment/search?ids={id}".format(
                id=comment["id"]
            )
        )
        data = response.json()["data"]
        if len(data) == 1:
            rev_comment = data[0]
            comment["author"] = rev_comment["author"]
            comment["body"] = rev_comment["body"]
            comment["score"] = rev_comment["score"]
            comment["recovered"] = True
            logging.info(f"Recovered {comment.get('id', '')} from pushshift")
    except Exception as e:
        logging.error(e)
    return comment


# Recover deleted comments via pushshift
def recover_deleted_comments(post):
    for comment in post["comments"]:
        if comment["body"] == "[deleted]":
            comment = recover_deleted_comment(comment)
        for reply in comment["replies"]:
            if reply["body"] == "[deleted]":
                reply = recover_deleted_comment(reply)
    return post


# Convert links to comments into posts
def handle_comment_links(comment):
    # Filter out root posts
    if comment.get("parent_id") is None:
        return comment

    comment["title"] = "Comment on " + comment["submission_title"]
    comment["savedcomment"] = comment["id"]
    comment["id"] = comment["submission"]
    comment["comments"] = comment["replies"]
    comment["selftext"] = comment["body"]
    comment[
        "permalink"
    ] = "https://www.reddit.com/r/{subreddit}/comments/{submission}/{title}/{id}".format(
        subreddit=comment["subreddit"],
        submission=comment["submission"],
        title=comment["title"],
        id=comment["id"],
    )
    return comment



import logging
import os
from pathlib import Path

import reddit2pdf.library as library

logger = logging.getLogger(__name__)


# Input: Path to directory containing HTML, media, and CSSfiles
# Output: PDF File path
# Note: HTML sources from same directory.
def generate_pdf_file(html_dir: str) -> Path:
    html_path = Path(html_dir)
    pdf_path = html_path / "output.pdf"

    # Add the HTML file and output PDF file to the command
    html_files = [f for f in html_path.glob("*.html")]
    logging.debug(f"Found HTML files: {html_files}")
    if len(html_files) != 1:
        raise ValueError("There should be exactly one HTML file in the directory.")

    # wkhtmltopdf --enable-local-file-access <html_file> test.pdf
    cmd = f"wkhtmltopdf --enable-local-file-access \
        --margin-bottom 0 --margin-left 0 --margin-right 0 --margin-top 0 \
        {html_files[0]} {pdf_path}"

    # Run the command with logging
    library.run_command(cmd)

    # Check that the PDF file exists in the directory
    assert os.path.exists(pdf_path), f"PDF file '{pdf_path}' does not exist"

    return pdf_path



import logging
import os

import reddit2pdf.library as library


def download_reddit_url(url: str, download_dir: str = None) -> str:
    """
    Downloads a Reddit URL using bulk-downloader-for-reddit and saves it to the specified directory.

    :param url: The Reddit URL to download.
    :param dir: The directory where the downloaded files should be saved.
    """
    # Create directory if required
    if not os.path.exists(download_dir):
        logging.info(f"Directory did not exist. Creating {download_dir}")
        os.mkdir(download_dir)

    assert library.is_reddit_url(url), f"Url was not a reddit URL: {url}"

    # BDFR only takes http start
    if not url.startswith("http://") and not url.startswith("https://"):
        url = "https://" + url

    # Define the command to execute
    # Primarily running on macos
    # Set format to JSON for easy python loading
    # Adust filename for readability
    # Required separate log. Assumes no download to same download_dir simultaneously
    # Adjust default folder scheme since we dont care about nested folders
    # NULL folder scheme to prevent subdirs
    cmd = f"bdfr clone {download_dir} --filename-restriction-scheme linux \
        --authenticate --format json --file-scheme {{SUBREDDIT}}_{{POSTID}} \
        --link {url} --log {os.path.join(download_dir, 'bdfr.log')} --folder-scheme ''"

    # Run the command with logging
    library.run_command(cmd)

    # Check that the JSON file exists in the directory
    json_file = os.path.join(
        download_dir, f"{url.split('/')[-5]}_{url.split('/')[-3]}.json"
    )
    assert os.path.exists(json_file), f"JSON file '{json_file}' does not exist"

    return download_dir



import logging
import os
import pkgutil
import shutil
import subprocess
import time

import jinja2
import markdown

logger = logging.getLogger(__name__)

templateLoader = jinja2.ChoiceLoader(
    [
        jinja2.FileSystemLoader(searchpath="./reddit2pdf/templates"),
        jinja2.PackageLoader("reddit2pdf", "templates"),
    ]
)
templateEnv = jinja2.Environment(loader=templateLoader)
templateEnv.add_extension("jinja2.ext.debug")
templateEnv.filters["markdown"] = markdown.markdown


def float_to_datetime(value):
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(value))


templateEnv.filters["float_to_datetime"] = float_to_datetime


# Copy media from the input folder to the file structure of the html pages
def copy_media(source_path, output_path):
    if output_path.endswith("mp4"):
        try:
            # This fixes mp4 files that won't play in browsers
            command = [
                "ffmpeg",
                "-nostats",
                "-loglevel",
                "0",
                "-i",
                source_path,
                "-c:v",
                "copy",
                "-c:a",
                "copy",
                "-y",
                output_path,
            ]
            logging.debug(f"Running {command}")
            subprocess.call(command)
        except Exception as e:
            logging.error("FFMPEG failed: " + str(e))
    else:
        shutil.copyfile(source_path, output_path)
    logging.debug(f"Moved {source_path} to {output_path}")


# Search the input folder for media files containing the id value from an archive
def find_matching_media(post, input_folder, output_folder):
    paths = []

    for dirpath, dnames, fnames in os.walk(output_folder):
        for f in fnames:
            if post["id"] in f and not f.endswith(".json") and not f.endswith(".html"):
                logging.debug(f"Existing media found: {os.path.join(dirpath + f)}")
                paths.append(f)
    if len(paths) > 0:
        logging.debug(f"Existing media found for {post['id']} = {paths}")
        post["paths"] = paths
        return post
    for dirpath, dnames, fnames in os.walk(input_folder):
        for f in fnames:
            if post["id"] in f and not f.endswith(".json"):
                logging.debug(f"New matching media found: {os.path.join(dirpath + f)}")
                copy_media(os.path.join(dirpath, f), os.path.join(output_folder, f))
                paths.append(f)
    post["paths"] = paths
    return post


# Creates the html for a post using the jinja2 template and writes it to a file
def write_post_to_file(post, output_folder):
    template = templateEnv.get_template("page.html")
    post["filename"] = post["id"] + ".html"
    post["filepath"] = os.path.join(output_folder, post["id"] + ".html")

    with open(post["filepath"], "w", encoding="utf-8") as file:
        file.write(template.render(post=post))
    logging.debug(f"Wrote {post['filepath']}")


def populate_css_file(output):
    css_output_path = os.path.join(output, "style.css")
    css_input_path = "./templates/style.css"
    if os.path.exists(css_input_path):
        shutil.copyfile(css_input_path, css_output_path)
    else:
        try:
            data = pkgutil.get_data(__name__, "templates/style.css")
            with open(css_output_path, "wb") as file:
                file.write(data)
        except Exception as e:
            logger.error(e)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging

import click
import requests

from reddit2pdf import __version__
from reddit2pdf.configuration import Configuration
from reddit2pdf.reddit2pdf import reddit2pdf

logger = logging.getLogger(__name__)


def _check_version(context, param, value):
    if not value or context.resilient_parsing:
        return
    current = __version__
    latest = requests.get("https://pypi.org/pypi/reddit2pdf/json").json()["info"][
        "version"
    ]
    print(f"You are currently using v{current} the latest is v{latest}")
    context.exit()


@click.group()
@click.help_option("-h", "--help")
@click.option(
    "--version",
    is_flag=True,
    is_eager=True,
    expose_value=False,
    callback=_check_version,
    help="Check version and exit.",
)
def cli():
    """reddit2pdf is used to download and archive content from Reddit."""
    pass


@cli.command("pdf")
@click.option("-o", "--outdir", type=str, required=True, help="")
@click.option("-l", "--link", default=None, type=str, required=True, help="")
@click.option("-v", "--verbose", default=None, count=True, help="")
@click.option("-u", "--unclean", is_flag=True, default=None, help="")
@click.option("--move-media", is_flag=True, default=None, help="")
@click.option("--move-html", is_flag=True, default=None, help="")
@click.help_option("-h", "--help")
@click.pass_context
def cli_pdf(context: click.Context, **_):
    """Archives a reddit thread as a pdf"""
    config = Configuration()
    config.process_click_arguments(context)
    reddit2pdf(config)


@cli.command("authenticate")
@click.help_option("-h", "--help")
def cli_authenticate(context: click.Context, **_):
    """Passthrough to bdfr authentication"""


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    cli()



import logging
import os
import shutil
import tempfile

import reddit2pdf.html2pdf as html2pdf
import reddit2pdf.json2html as json2html
import reddit2pdf.url2bdfr as url2bdfr
from reddit2pdf.configuration import Configuration

logger = logging.getLogger(__name__)


def _move_collateral(
    indir: str, outdir: str, include_media: bool = False, include_html: bool = False
):
    # Get a list of files in the input directory
    files = os.listdir(indir)

    # Loop through the files and move any pdfs to the output directory
    pdf_count = 0
    for file in files:
        if file.endswith(".pdf"):
            if os.path.exists(os.path.join(outdir, file)):
                logging.warning(
                    f"PDF file already existed in outdir: {outdir}. Replacing..."
                )
                os.remove(os.path.join(outdir, file))

            shutil.move(os.path.join(indir, file), os.path.join(outdir, file))
            pdf_count += 1
            logging.info(f"Moved PDF file {file} from {indir} to {outdir}")
        elif (file.endswith(".html") or file.endswith(".css")) and include_html:
            if os.path.exists(os.path.join(outdir, file)):
                logging.warning(
                    f"HTML file already existed in outdir: {outdir}. Replacing..."
                )
                os.remove(os.path.join(outdir, file))

            shutil.move(os.path.join(indir, file), os.path.join(outdir, file))
            logging.info(f"Moved HTL file {file} from {indir} to {outdir}")
        elif include_media:
            # If include_media is True, move any files with media extensions to the output directory
            media_extensions = [
                ".jpg",
                ".jpeg",
                ".png",
                ".gif",
                ".mp4",
                ".mov",
                ".avi",
                ".mp3",
                ".wav",
            ]
            if os.path.splitext(file)[1] in media_extensions:
                file_path = os.path.join(indir, file)
                file_size_mb = os.path.getsize(file_path) / (1024 * 1024)
                if file_size_mb > 50:
                    logging.warning(
                        f"Media file {file} is larger than 50 megabytes ({file_size_mb:.2f} MB)"
                    )

                if os.path.exists(os.path.join(outdir, file)):
                    logging.debug(
                        f"Media file already existed in output. Skipping {file}"
                    )
                else:
                    shutil.move(file_path, os.path.join(outdir, file))
                    logging.info(f"Moved media file {file} from {indir} to {outdir}")

    # Check that exactly one PDF file was moved
    if pdf_count != 1:
        raise ValueError(
            f"{pdf_count} PDF files were moved. Please ensure that exactly one PDF file is present in {indir}."
        )


def reddit2pdf(conf: Configuration):
    # Generate temporary directory to store all bdfr + html intermediaries
    tempdir = tempfile.mkdtemp(suffix="reddit2pdf", dir="/tmp")
    # Use BDFR to download the reddit URL to the temp directory
    bdfr_dir = url2bdfr.download_reddit_url(conf.link, tempdir)
    # Generate HTML file from resulting BDFR archive
    html_dir = json2html.generate_reddit_post_html(bdfr_dir)
    # Generate PDF file from HTML file
    pdf_path = html2pdf.generate_pdf_file(html_dir)
    logging.info(f"Generated PDF: {pdf_path}")

    if not os.path.exists(conf.outdir):
        logging.info(f"Outdir {conf.outdir} did not exist. Creating...")
        os.makedirs(conf.outdir, exist_ok=True)

    # Move desired collateral to output directory
    _move_collateral(tempdir, conf.outdir, conf.move_media, conf.move_html)

    # Destroy temp directory
    if not conf.unclean:
        os.remove(tempdir)



import logging
import os
import shlex
import subprocess
from urllib.parse import urlparse

import requests

logger = logging.getLogger(__name__)


def run_command(
    cmd: str, cwd: str = None, ignore_errors: bool = False, is_dry_run: bool = False
):
    logging.info(f"Running  command: {cmd}")

    if is_dry_run:
        logging.info("Dry run mode. Skipping command execution")
        return ([], [], 0)

    try:
        process = subprocess.Popen(
            shlex.split(cmd),
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True,
            cwd=cwd,
        )

        stdout_lines = []
        stderr_lines = []

        # Stream stdout and stderr to Python logging and capture the output
        is_finished = False
        while True:
            for stdout_line in process.stdout.readlines():
                stdout_line = stdout_line.strip()
                logging.debug(stdout_line)
                stdout_lines.append(stdout_line)

            for stderr_line in process.stderr.readlines():
                stderr_line = stderr_line.strip()
                logging.debug(stderr_line)
                stderr_lines.append(stderr_line)

            if is_finished:
                break
            if process.poll() is not None:
                is_finished = True

        exit_code = process.returncode
        output = (stdout_lines, stderr_lines, exit_code)
        return output
    except Exception as e:
        logging.error(f"Error occurred: {e}")
        if ignore_errors:
            logging.warning("Ignoring error and returning empty output.")
            return ([], [], 1)
        else:
            raise


def url_checker(domains):
    def decorator(func):
        def wrapper(url):
            # Handle RFC 1808
            if not url.startswith("http"):
                url = "https://" + url

            parsed_url = urlparse(url)
            domain = parsed_url.netloc
            logging.debug(f"Checking {url=} of {domain=} for {domains=}")
            return any(subdomain in domain for subdomain in domains)

        return wrapper

    return decorator


@url_checker(
    ["www.reddit.com", "www.redd.it", "www.redditmedia.com", "www.reddituploads.com"]
)
def is_reddit_url(url):
    """
    Check if the given URL is from Reddit.

    Args:
        url (str): The URL to check.

    Returns:
        bool: True if the URL is from Reddit, False otherwise.
    """
    pass


def is_url_accessible(url):
    """
    Check if a URL is accessible.

    Args:
        url (str): The URL to check.

    Returns:
        bool: True if the URL is accessible, False otherwise.
    """
    try:
        response = requests.head(url)
        is_accessible = response.status_code == 200
        logging.debug(f"{url=} was accessible: {is_accessible}")
        return is_accessible
    except requests.exceptions.RequestException:
        logging.warning(f"Was unable to access url: {url}")
        return False


def get_json_file_path(directory_path):
    if not os.path.exists(directory_path):
        raise FileNotFoundError(f"Unable to find directory: {directory_path}")

    json_files = [f for f in os.listdir(directory_path) if f.endswith(".json")]
    if len(json_files) != 1:
        raise ValueError("There should be exactly one JSON file in the directory.")
    return os.path.join(directory_path, json_files[0])


# Check for path, create if does not exist
def ensure_path_exists(path):
    path = os.path.join(path, "")
    dir = os.path.dirname(path)
    if not os.path.exists(dir):
        os.makedirs(dir)
        logging.debug(f"Created {dir}")
    return dir



import json
import logging
import os

import reddit2pdf.bdfrjsonhelper as bdfrjsonhelper
import reddit2pdf.htmlwriter as htmlwriter
import reddit2pdf.library as library

logger = logging.getLogger(__name__)


# Input: Path to BDFR download directory containing the JSON file + any media
# Output: Path to temporary directory containing HTML + media (should be the same folder)
def generate_reddit_post_html(bdfr_json_dir: str) -> str:
    if not os.path.exists(bdfr_json_dir):
        raise ValueError(f"BDFR directory did not exist: {bdfr_json_dir}")

    bdfr_json_path = library.get_json_file_path(bdfr_json_dir)

    with open(bdfr_json_path) as fh:
        post = json.load(fh)
        logging.debug(f"Loaded {bdfr_json_path}")

        if post.get("id") is not None:
            logging.debug(f"Imported  {bdfr_json_path}")

        try:
            post = bdfrjsonhelper.handle_comment_links(post)
            post = bdfrjsonhelper.recover_deleted_comments(post)
            post = bdfrjsonhelper.recover_deleted_post(post)
            post = bdfrjsonhelper.get_sub_from_post(post)

            # Move + log media
            post = htmlwriter.find_matching_media(post, bdfr_json_dir, bdfr_json_dir)

            # Create HTML collateral
            htmlwriter.write_post_to_file(post, bdfr_json_dir)
            htmlwriter.populate_css_file(bdfr_json_dir)
        except Exception as e:
            logging.error(f"Processing post {post['id']} has failed due to: {str(e)}")

    return bdfr_json_dir
